%INPUT: Two edges in the form of a nx3 matrix. The type option has no been
    %implemented yet.
%FUNCTION: Calculates the surface area of overlap and space between two 
    %assembled edges
%OUTPUT: The calculated surface area value


function area = InterferenceDetection(edge1,edge2,type)

%CALCULATING DISTANCE MATRIX
distanceMatrix     = zeros(length(edge1(:,1)),length(edge2(:,2)));
intersectionPoints = [];
for i=1:length(edge1)
    for j=1:length(edge2)
        distanceMatrix(i,j) = sqrt((edge1(i,1)-edge2(j,1))^2+(edge1(i,2)-edge2(j,2))^2+(edge1(i,3)-edge2(j,3))^2);
    end
end

%FINDING INTERSECTION POINTS
cnt = 0;
for i=1:length(edge1)
    [distance, index] = min(distanceMatrix(i,:));
    if distance<0.015
        cnt = cnt+1;
        intersectionPoints(cnt,1) = i;
        intersectionPoints(cnt,2) = index;
    end
end

%CHECKING INITIAL AND FINAL POINTS OF INTERSECTION POINTS
if isempty(intersectionPoints)==0
    if length(intersectionPoints(:,1))>1
        if type=='e'
            if intersectionPoints(2,1)>1
                [distance, index] = min(distanceMatrix(1,:));
                intersectionPoints(1,1) = 1;
                intersectionPoints(1,2) = index;
            else
                intersectionPoints(1,:) = intersectionPoints(2,:);
            end
            if intersectionPoints(end,1)<length(edge1)
                [distance, index] = min(distanceMatrix(length(edge1),:));
                intersectionPoints(end+1,1) = length(edge1);
                intersectionPoints(end,2)    = index;
            end
        end
    end
end

%COMPUTING INTERFERENCE
area   = 0;
if isempty(intersectionPoints)==0
    for i=1:length(intersectionPoints(:,1))-1
        
        %                 figure(1); hold on
        %                 plot3(edge1(:,1),edge1(:,2),edge1(:,3))
        %                 plot3(edge2(:,1),edge2(:,2),edge2(:,3),'r')
        %                 plot3(edge1(intersectionPoints(i,1),1),edge1(intersectionPoints(i,1),2),edge1(intersectionPoints(i,1),3),'.b');
        %                 plot3(edge2(intersectionPoints(i,2),1),edge2(intersectionPoints(i,2),2),edge2(intersectionPoints(i,2),3),'.r');
        
        plane = [];
        plane = edge1(intersectionPoints(i,1):intersectionPoints(i+1,1),:);
        for j=intersectionPoints(i+1,2):intersectionPoints(i,2)
            plane(end+1,:) = edge2(j,:);
        end
        plane(end+1,:) = plane(1,:);
        
        %                 figure(2); hold on
        %                 plot(plane(:,1),plane(:,2));
        %                 plot(plane(:,1),plane(:,2),'.r');
        
        
        plane = rotate(plane);
        area  = polyarea(plane(:,1),plane(:,2))+area;
    end
end

end

%%