%INPUT: the following
    %edge1: edge belonging to piece1
    %edge2: edge belonging to piece2
    %piece1: puzzle piece 
    %piece2: puzzle piece
    %oldEdge1: belongs to edge1, not used in this version of the program
    %oldEdge2: belongs to edge2, not used in this version of the program
%FUNCTION: Assembles matching edges together
%OUTPUT: edge2 is rigidly transformed to fit into edge1

function output=PutTogetherAdvanced(edge1,edge2,piece1,piece2,oldEdge1,oldEdge2)

%TEST IF LENGTH OF THE EDGES IS SIGNIFICANTLY DIFFERENT
distances = zeros(2,1);
for i=2:length(edge1(:,1))
    distances(1) = sqrt((edge1(i,1)-edge1(i-1,1))^2+(edge1(i,2)-edge1(i-1,2))^2+(edge1(i,3)-edge1(i-1,3))^2)+distances(1);
    distances(2) = sqrt((edge2(i,1)-edge2(i-1,1))^2+(edge2(i,2)-edge2(i-1,2))^2+(edge2(i,3)-edge2(i-1,3))^2)+distances(2);
end
distances = sort(distances);

%TRANSFORMATION MATRICES
rigidTransformation1 = estimateRigidTransform(edge1',fliplr(edge2'));
rotationMatrix1      = rigidTransformation1(1:3,1:3);
translation1        = rigidTransformation1(1:3,4);

rigidTransformation2 = estimateRigidTransform(edge1',edge2');
rotationMatrix2      = rigidTransformation2(1:3,1:3);
translation2         = rigidTransformation2(1:3,4);

%ROTATING PIECES
shiftedPiece2a = zeros(length(piece2),3);
shiftedPiece2b = zeros(length(piece2),3);
for a=1:length(piece2)
    shiftedPiece2a(a,:)=(rotationMatrix1*(piece2(a,:))'+translation1)';
end
for a=1:length(piece2)
    shiftedPiece2b(a,:)=(rotationMatrix2*(piece2(a,:))'+translation2)';
end

%DETERMINING BETTER ROTATION MATRIX
shiftedEdge2a  = zeros(length(edge2),3);
shiftedEdge2b  = zeros(length(edge2),3);
shiftedOldEdge2 = zeros(length(oldEdge2),3);
for j=1:length(edge2(:,1))
    shiftedEdge2a(j,:)   = (rotationMatrix1*(edge2(j,:))'+translation1)';
end
for j=1:length(edge2(:,1))
    shiftedEdge2b(j,:)   = (rotationMatrix2*(edge2(j,:))'+translation2)';
end
for j=1:length(oldEdge2(:,1))
    shiftedOldEdge2(j,:) = (rotationMatrix1*(oldEdge2(j,:))'+translation1)';
end

%OUTPUT
if HausdorffDist(edge1,shiftedEdge2a)>0.075 
    %     fprintf('FAIL: THE HAUS IS %f \n',HausdorffDist(edge1,shiftedEdge2a));
    %     figure(5); hold on
    %     plot3(piece1(:,1),piece1(:,2),piece1(:,3),'k','LineWidth',2);
    %     plot3(shiftedPiece2a(:,1),shiftedPiece2a(:,2),shiftedPiece2a(:,3),'LineWidth',2);
    %     plot3(edge1(:,1),edge1(:,2),edge1(:,3),'.g','LineWidth',2);
    %     plot3(shiftedEdge2a(:,1),shiftedEdge2a(:,2),shiftedEdge2a(:,3),'.r','LineWidth',2);
    %     fprintf('%d and %d\n',out1,out2);
    %     fprintf('ARC LENGTH: %f\n INTERFERENCE: %f \n RATIO: %f',sum(distances),InterferenceDetection(edge1,shiftedEdge2a),InterferenceDetection(edge1,shiftedEdge2a)/sum(distances));
    
    output ='w';
else
    %     fprintf('PASS: THE HAUS IS %f \n',HausdorffDist(edge1,shiftedEdge2a));
    %     figure(5); hold on
    %     plot3(piece1(:,1),piece1(:,2),piece1(:,3),'k','LineWidth',2);
    %     plot3(shiftedPiece2a(:,1),shiftedPiece2a(:,2),shiftedPiece2a(:,3),'LineWidth',2);
    %     plot3(edge1(:,1),edge1(:,2),edge1(:,3),'.g','LineWidth',2);
    %     plot3(shiftedEdge2a(:,1),shiftedEdge2a(:,2),shiftedEdge2a(:,3),'.r','LineWidth',2);
    %     fprintf('ARC LENGTH: %f\n INTERFERENCE: %f \n RATIO: %f',sum(distances),InterferenceDetection(edge1,shiftedEdge2a),InterferenceDetection(edge1,shiftedEdge2a)/sum(distances));

    output = rigidTransformation1;
end

% %PLOTTING PIECES
% figure(5); hold on
% plot3(piece1(:,1),piece1(:,2),piece1(:,3),'k','LineWidth',2);
% plot3(shiftedPiece2a(:,1),shiftedPiece2a(:,2),shiftedPiece2a(:,3),'LineWidth',2);
% plot3(edge1(:,1),edge1(:,2),edge1(:,3),'.g','LineWidth',2);
% plot3(shiftedEdge2a(:,1),shiftedEdge2a(:,2),shiftedEdge2a(:,3),'.r','LineWidth',2);

% figure(2); hold on
% plot3(piece1(:,1),piece1(:,2),piece1(:,3),'k','LineWidth',2);
% plot3(p2b(:,1),p2b(:,2),p2b(:,3),'LineWidth',2);
% plot3(c1(:,1),c1(:,2),c1(:,3),'.r','LineWidth',2);
end

%%
