function output=PutTogetherAdvanced2(edge1,edge2,piece1,piece2,oldEdge1,oldEdge2)

%TEST IF LENGTH OF THE EDGES IS SIGNIFICANTLY DIFFERENT
distances = zeros(2,1);
for i=2:length(edge1(:,1))
    distances(1) = sqrt((edge1(i,1)-edge1(i-1,1))^2+(edge1(i,2)-edge1(i-1,2))^2+(edge1(i,3)-edge1(i-1,3))^2)+distances(1);
    distances(2) = sqrt((edge2(i,1)-edge2(i-1,1))^2+(edge2(i,2)-edge2(i-1,2))^2+(edge2(i,3)-edge2(i-1,3))^2)+distances(2);
end

%TRANSFORMATION MATRICES
rigidTransformation1 = estimateRigidTransform(edge1',fliplr(edge2'));
rotationMatrix1      = rigidTransformation1(1:3,1:3);
translation1         = rigidTransformation1(1:3,4);

rigidTransformation2 = estimateRigidTransform(edge1',edge2');
rotationMatrix2      = rigidTransformation2(1:3,1:3);
translation2         = rigidTransformation2(1:3,4);

%ROTATING PIECES
shiftedPiece2a = zeros(length(piece2),3);
shiftedPiece2b = zeros(length(piece2),3);
for a=1:length(piece2)
    shiftedPiece2a(a,:)=(rotationMatrix1*(piece2(a,:))'+translation1)';
end
for a=1:length(piece2)
    shiftedPiece2b(a,:)=(rotationMatrix2*(piece2(a,:))'+translation2)';
end

%DETERMINING BETTER ROTATION MATRIX
shiftedEdge2a   = zeros(length(edge2),3);
shiftedEdge2b   = zeros(length(edge2),3);
shiftedOldEdge2 = zeros(length(oldEdge2),3);
for j=1:length(edge2(:,1))
    shiftedEdge2a(j,:)   = (rotationMatrix1*(edge2(j,:))'+translation1)';
end
for j=1:length(edge2(:,1))
    shiftedEdge2b(j,:)   = (rotationMatrix2*(edge2(j,:))'+translation2)';
end
for j=1:length(oldEdge2(:,1))
    shiftedOldEdge2(j,:) = (rotationMatrix1*(oldEdge2(j,:))'+translation1)';
end

%DETERMINING CONCAVITY/CONVEXITY OF EDGES
projectedPiece1          = rotate(piece1);
totalArea1               = polyarea(projectedPiece1(:,1),projectedPiece1(:,2));
temp1                    = setdiff(piece1,edge1,'rows','stable');
projectedPiece1MinusEdge = setdiff(temp1,oldEdge1,'rows','stable');
projectedPiece1MinusEdge = rotate(projectedPiece1MinusEdge);
partionedArea1           = polyarea(projectedPiece1MinusEdge(:,1),projectedPiece1MinusEdge(:,2));

if totalArea1<partionedArea1
    out1 = 1;
else
    out1 = -1;
end

projectedPiece2           = rotate(shiftedPiece2a);
totalArea2                = polyarea(projectedPiece2(:,1),projectedPiece2(:,2));
temp2                     = setdiff(shiftedPiece2a,shiftedEdge2a,'rows','stable');
projectedPiece2MinusEdge  = setdiff(temp2,shiftedOldEdge2,'rows','stable');
projectedPiece2MinusEdge  = rotate(projectedPiece2MinusEdge);
partionedArea2            = polyarea(projectedPiece2MinusEdge(:,1),projectedPiece2MinusEdge(:,2));

if totalArea2<partionedArea2
    out2 = 1;
else
    out2 = -1;
end

%OUTPUT
if HausdorffDist(edge1,shiftedEdge2a)>0.08 || out1==out2
%         fprintf('FAIL: THE HAUS IS %f \n',HausdorffDist(edge1,shiftedEdge2a));
%         figure(5); hold on
%         plot3(piece1(:,1),piece1(:,2),piece1(:,3),'k','LineWidth',2);
%         plot3(shiftedPiece2a(:,1),shiftedPiece2a(:,2),shiftedPiece2a(:,3),'LineWidth',2);
%         plot3(edge1(:,1),edge1(:,2),edge1(:,3),'.g','LineWidth',2);
%         plot3(shiftedEdge2a(:,1),shiftedEdge2a(:,2),shiftedEdge2a(:,3),'.r','LineWidth',2);
        %fprintf('%d and %d\n',out1,out2);
        %fprintf('ARC LENGTH: %f\n INTERFERENCE: %f \n RATIO: %f \n',mean(distances),InterferenceDetection(edge1,shiftedEdge2a,'e'),InterferenceDetection(edge1,shiftedEdge2a,'e')/mean(distances));
    
    output = 'w';
else
%         fprintf('PASS: THE HAUS IS %f \n',HausdorffDist(edge1,shiftedEdge2a));
%         figure(5); hold on
%         plot3(piece1(:,1),piece1(:,2),piece1(:,3),'k','LineWidth',2);
%         plot3(shiftedPiece2a(:,1),shiftedPiece2a(:,2),shiftedPiece2a(:,3),'LineWidth',2);
%         plot3(edge1(:,1),edge1(:,2),edge1(:,3),'.g','LineWidth',2);
%         plot3(shiftedEdge2a(:,1),shiftedEdge2a(:,2),shiftedEdge2a(:,3),'.r','LineWidth',2);
        %fprintf('%d and %d\n',out1,out2);
    
    interference = InterferenceDetection(edge1,shiftedEdge2a,'e');
   %fprintf('ARC LENGTH: %f\n INTERFERENCE: %f \n RATIO: %f \n',mean(distances),interference,interference/mean(distances));
    
    if interference/mean(distances)<0.01
        output = rigidTransformation1;
    else
        output = 'w';
    end
end

end

%%
