%THIS FUNCTION CALCULATES THE SIMILARITY BETWEEN TWO SEGMENTS OF A
%SIGNATURE CURVES, WHICH IS THE INPUT. THE OUTPUT IS A SKEWNESS VALUE WHICH
%IS GREATER THAN ZERO AND THE SIMILARITY MEASURE.

function skew = compareEgg3D(sink,source)
% figure(1); hold on
% plot3(sink(:,1),sink(:,2),sink(:,3));
% plot3(sink(:,1),sink(:,2),sink(:,3),'.b');
% plot3(source(:,1),source(:,2),source(:,3),'r');
% plot3(source(:,1),source(:,2),source(:,3),'.r');
% plot3(sink(1,1),sink(1,2),sink(1,3),'.g');
% plot3(source(1,1),source(1,2),source(1,3),'.g');

%SIMILARITY CALCULATION 
for t=1:8  
    %DETERMINING INITIAL AND FINAL POINTS
    if t==5
        for i=1:length(sink(:,1))
            sink(i,2) = -sink(i,2);
        end
    end
    if t==1 || t==5
        seg1 = sink;
        seg2 = source;    
    elseif t==2 || t==6
        seg1 = sink;
        seg2 = (fliplr(source'))';
    elseif t==3 || t==7
        seg1 = fliplr(sink')';
        seg2 = source;    
    else
        seg1 = fliplr(sink')';
        seg2 = (fliplr(source'))';
    end
    
%     if t==1
%         figure(2); hold on
%         plot3(seg1(:,1),seg1(:,2),seg1(:,3));
%         plot3(seg1(:,1),seg1(:,2),seg1(:,3),'.b');
%         plot3(seg2(:,1),seg2(:,2),seg2(:,3),'k');
%         plot3(seg2(:,1),seg2(:,2),seg2(:,3),'.k');
%         
%         plot3(seg1(1,1),seg1(1,2),seg1(1,3),'.g');
%         plot3(seg2(1,1),seg2(1,2),seg2(1,3),'.g');
%         plot3(seg1(50,1),seg1(50,2),seg1(50,3),'.c');
%         plot3(seg2(50,1),seg2(50,2),seg2(50,3),'.c');
%         plot3(seg1(end,1),seg1(end,2),seg1(end,3),'.r');
%         plot3(seg2(end,1),seg2(end,2),seg2(end,3),'.r');
%     end
    
    %CONVERSION TO SPHERICAL COORDINATES
    polar1=zeros(length(seg1),3); polar2=zeros(length(seg1),3); 
    for i=1:length(seg1)
        [theta,phi,rho] = cart2sph(seg1(i,1),seg1(i,2),seg1(i,3));
        if phi<0
            phi = -phi+pi/2;
        else
            phi = pi/2-phi;
        end
        polar1(i,1)     = rho;
        polar1(i,2)     = phi;
        polar1(i,3)     = theta;
    end
    for i=1:length(seg2)
        [theta,phi,rho] = cart2sph(seg2(i,1),seg2(i,2),seg2(i,3));
        if phi<0
            phi = -phi+pi/2;
        else
            phi = pi/2-phi;
        end
        polar2(i,1)     = rho;
        polar2(i,2)     = phi;
        polar2(i,3)     = theta;
    end 
    
    %CUMULATIVE CALCULATION
    for i=2:length(polar1)
        polar1(i,1) = polar1(i,1)+polar1(i-1,1);
        polar1(i,2) = polar1(i,2)+polar1(i-1,2);
        polar1(i,3) = polar1(i,3)+polar1(i-1,3);
    end
    for i=2:length(seg2)
        polar2(i,1) = polar2(i,1)+polar2(i-1,1);
        polar2(i,2) = polar2(i,2)+polar2(i-1,2);
        polar2(i,3) = polar2(i,3)+polar2(i-1,3);
    end
    
    for i=1:length(polar1(:,1))
        polar1(i,1) = -polar1(i,1);
        polar1(i,2) = -polar1(i,2);
        polar1(i,3) = -polar1(i,3);
    end
    
    %COMPILING CUMULATIVE VECTORS
    distVector  = polar1(:,1);   distVector(length(polar1(:,1))+1:length(polar1(:,1))+length(polar2(:,1)))  = polar2(:,1);
    thetaVector = polar1(:,2);   thetaVector(length(polar1(:,2))+1:length(polar1(:,2))+length(polar2(:,2))) = polar2(:,2);
    phiVector   = polar1(:,3);   phiVector(length(polar1(:,3))+1:length(polar1(:,3))+length(polar2(:,3)))   = polar2(:,3);
    
    %CALCULATING SKEW
    mean         = 0;
    skewtheta    = ((1/length(thetaVector))*sum((thetaVector-mean).^3))/((1/length(thetaVector))*sum((thetaVector-mean).^2))^(1.5);
    skewphi      = ((1/length(phiVector))*sum((phiVector-mean).^3))/((1/length(phiVector))*sum((phiVector-mean).^2))^(1.5);
    skew         = ((1/length(distVector))*sum((distVector-mean).^3))/((1/length(distVector))*sum((distVector-mean).^2))^(1.5);
    tempPhi(t)   = skewphi;
    tempTheta(t) = skewtheta;
    tempDist(t)  = skew;
    temp(t)      = abs(skew)+abs(skewphi)+abs(skewtheta);

   %fprintf('THE SKEWS ARE %f AND %f AND %f\n',skew,skewphi,skewtheta);
end

out  = sort(temp);

outPhi   = sort(tempPhi);
outTheta = sort(tempTheta);
outDist  = sort(tempDist);


skew = out(1);
skew = min(outPhi)+min(outTheta)+min(outDist);
end

%%
