%INPUT IS A PUZZLE

function out = SyntheticSolver(x)

%SCRAMBLER
for h=1:length(x)
    x{h,1} = scrambler3d(x{h,1});
end

%SEGMENTING
fprintf('PARTITIONING SIGNATURE INTO BIVERTEX ARCS\n');
puzzle = KappaSegmentation(x);

figure(1); hold on
for i=1:length(x)
plot3(puzzle(i).piece(:,1),puzzle(i).piece(:,2),puzzle(i).piece(:,3),'r','LineWidth',2);
end

%CALCULATING SIMILARITY SCORE
%score is stored so that first entry is matching piece, second is matching
%edge, and third entry is the similarity score
fprintf('SIMILARITY SCORE CALCULATION\n');
num = 0;
for h=1:length(x) %SET PIECE
    if (isempty(puzzle(h).edge))
        continue
    end
    for j=1:length(puzzle(h).edge(:,1)) %SET EDGE
        cnt = 0; 
        for k=1:length(x) %COMPARATIVE PIECE
            if h==k
                continue
            end
            if isempty(puzzle(k).edge)
                continue
            end
            for m=1:length(puzzle(k).edge(:,1)) %COMPARATIVE EDGE
                skew                       = compare3D(puzzle(k).edge{m,2},puzzle(h).edge{j,2});
                cnt                        = cnt+1;
                puzzle(h).edge{j,3}(cnt,1) = skew;
                puzzle(h).edge{j,3}(cnt,2) = k; 
                puzzle(h).edge{j,3}(cnt,3) = m;
                
                if (h<k)
                    num               = num+1;
                    skewMatrix(num,1) = skew;
                    skewMatrix(num,2) = h;
                    skewMatrix(num,3) = j;
                    skewMatrix(num,4) = k;
                    skewMatrix(num,5) = m;
                end          
            end
        end
        puzzle(h).edge{j,3} = sortrows(puzzle(h).edge{j,3},1);
    end
end
skewMatrix = sortrows(skewMatrix,1);

%%%%%%%%%%%%%%%%%%%%%
%%% PRIMARY SOLVE %%%
%%%%%%%%%%%%%%%%%%%%%
fprintf('PRIMARY SOLVE\n');

threshold = 0.1;
increment = 0.05;

%CREATING SOLUTION STRUCTURE
for h=1:length(x)
    sol(h).pieces=h; sol(h).match=[];
    if (isempty(puzzle(h).edge)==0)
        for i=1:length(puzzle(h).edge(:,1))
            sol(h).edge{i,1} = zeros(1,2);
            sol(h).edge{i,2} = zeros(1,2);
        end
    end
end
%SEARCHING FOR OBVIOUS MATCHES
for h=1:length(x)
    if (isempty(puzzle(h).edge)==0)
        for i=1:length(puzzle(h).edge(:,1))
            if (puzzle(h).edge{i,3}(1,1)<threshold)
                p1 = h; p2 = puzzle(h).edge{i,3}(1,2);
                e1 = i; e2 = puzzle(h).edge{i,3}(1,3);
                
                %FILLING THE PIECE
                sol(h).edge{e1,1}(1,1)  = p1;
                sol(h).edge{e1,1}(1,2)  = e1;
                sol(h).edge{e1,2}(1)    = p2;
                sol(h).edge{e1,2}(2)    = e2;
                %FILLING THE OTHER PIECE
                sol(p2).edge{e2,1}(1,1) = p2;
                sol(p2).edge{e2,1}(1,2) = e2;
                sol(p2).edge{e2,2}(1)   = p1;
                sol(p2).edge{e2,2}(2)   = e1;
                %FILLING EDGE DATABASE
                spot = length(sol(h).match)+1;  sol(h).match(spot)  = p2;
                here = length(sol(p2).match)+1; sol(p2).match(here) = h;            
            end
        end
    end
end

%DELETING DUPLICATES
for h=1:length(puzzle)
    sol(h).match = unique(sol(h).match);
end

%%%%%%%%%%%%%%%%%%%%%%%%
%%% PRIMARY ASSEMBLY %%%
%%%%%%%%%%%%%%%%%%%%%%%%
clusters                    = [];
moved                       = zeros(length(x),1);
[puzzle,sol,moved,clusters] = AssemblyRevise(puzzle,sol,moved,clusters);

% figure(2); hold on
% for i=1:length(x)
%     if moved(i)==1
%         plot3(puzzle(i).piece(:,1), puzzle(i).piece(:,2),puzzle(i).piece(:,3),'LineWidth',2);
%     end
% end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%SEARCHING FOR ARBITRARY MATCHES%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf('GENERAL SOLVE\n');

%GENERAL SOLVE
for V=1:5
    for v=1:length(x)
        %CHECKING IF PIECE IS ALREADY FULLY SOLVED
        if length(clusters{1,1})==length(puzzle)
            break
        end
        
        %DETERMINING TRIAL EDGE
        p1 = v;
        for i=1:length(puzzle(p1).edge(:,1))
            if (sol(p1).edge{i,1}(1,1)==0)
                for j=1:length(puzzle(p1).edge{i,3}(:,1))
                    if puzzle(p1).edge{i,3}(j,1)>(threshold+increment)
                        break
                    end
                    e1  = i;
                    e2  = puzzle(p1).edge{i,3}(j,3);
                    p2  = puzzle(p1).edge{i,3}(j,2);
                    
                    %CHECKING IF EDGE IS ALREADY SOLVED
                    if (sol(p2).edge{e2,1}(1)~=0)
                        continue
                    end
                    
                     %CHECKING IF OTHER PIECE HAS MULTIPLE GOOD MATCHES
                    cnt    = 1;
                    update = 1;
                    while puzzle(p2).edge{e2,3}(cnt,2)~=p1 && puzzle(p2).edge{e2,3}(cnt,3)~=e1
                        cnt = cnt+1;
                    end
                    
                    if cnt==1 
                        closePiece = puzzle(p2).edge{e2,3}(2,2);
                        closeEdge  = puzzle(p2).edge{e2,3}(2,3);
                        if puzzle(p2).edge{e2,3}(2,1)<threshold && sol(closePiece).edge{closeEdge,1}(1)==0
                            update = 0;
                        end
                    elseif cnt>1 
                        closePiece1 = puzzle(p2).edge{e2,3}(cnt-1,2);
                        closeEdge1  = puzzle(p2).edge{e2,3}(cnt-1,3);
                        closePiece2 = puzzle(p2).edge{e2,3}(cnt+1,2);
                        closeEdge2  = puzzle(p2).edge{e2,3}(cnt+1,3);
                        
                        if puzzle(p2).edge{e2,3}(cnt-1,1)<threshold+increment  && sol(closePiece1).edge{closeEdge1,1}(1)==0
                            update = 0;
                        elseif puzzle(p2).edge{e2,3}(cnt+1,1)<threshold+increment && sol(closePiece2).edge{closeEdge2,1}(1)==0
                            update = 0;
                        end
                    end
                    
                    %UPDATE SOLUTION DATABASES
                    if update==1
                    sol(p1).edge{e1,1}(1,1) = p1; sol(p1).edge{e1,1}(1,2) = e1;
                    sol(p1).edge{e1,2}(1,1) = p2; sol(p1).edge{e1,2}(1,2) = e2;
                    sol(p2).edge{e2,1}(1,1) = p2; sol(p2).edge{e2,1}(1,2) = e2;
                    sol(p2).edge{e2,2}(1,1) = p1; sol(p2).edge{e2,2}(1,2) = e1;
                    
                    %UPDATING SOLUTION STRUCTURE
                    spot = length(sol(p1).match)+1; sol(p1).match(spot) = p2;
                    here = length(sol(p2).match)+1; sol(p2).match(here) = p1;
                    end
                    break %BECAUSE THE EDGE IS A MATCH
                end
            else
                continue %BECAUSE PIECE ALREADY HAS A SOLUTION
            end
        end
    end
    threshold = threshold+increment;
    
    %ASSEMBLY
    [puzzle,sol,moved,clusters] = AssemblyRevise(puzzle,sol,moved,clusters);
%     [puzzle,sol,temp]           = UpdateSolution(puzzle,sol,clusters);
%     clear clusters
%     clusters                    = temp;
end

% figure(4); hold on
% for i=1:length(x)
%     if isempty(sol(i).match)==0
%         plot3(puzzle(i).piece(:,1), puzzle(i).piece(:,2),puzzle(i).piece(:,3),'LineWidth',2);
%     end
% end

%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%  SUBPUZZLE ASSEMBLY %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf('SUBPUZZLE ASSEMBLY');
solved = 0;

%DELETING DUPLICATES
for h=1:length(puzzle)
    sol(h).match = unique(sol(h).match);
end

%ADDING UNMATCHED PIECES AS THEIR OWN CLUSTER
for h=1:length(puzzle)
    if isempty(sol(h).match)
        clusters{length(clusters)+1,1} = h;
    end
end

for h=1:length(clusters)
    clusters{h,2} = length(puzzle)-length(clusters{h,1});
end

clusters      = sortrows(clusters,2);

%CHECKING IF PUZZLE IS FULLY SOLVED
for h=1:length(clusters)
    if length(clusters{h,1})==length(puzzle)
        solved = 1;
        break
    end
end

%DETERMINING SUBPUZZLE CLUSTERS
if solved==0
    for h=length(clusters):-1:2 %CLUSTER WE ARE LOOKING TO MATCH TO ANOTHER CLUSTER  (SMALL->BIG)
        if isempty(clusters{h,1})
            continue
        end
        cluster  = [];
        cluster  = clusters{h,1};
        for p=(h-1):-1:1     %CLUSTER WE ARE LOOKING INTO
            
            if p==h
                continue
            end
            
            %SEARCHING FOR BEST MATCH BETWEEN CLUSTERS
            for i=1:length(skewMatrix(:,1))
                if skewMatrix(i,1)>1
                    break
                end
                match    = 0;
                if ismember(skewMatrix(i,2),clusters{h,1}) && ismember(skewMatrix(i,4),clusters{p,1}) && skewMatrix(i,1)<1
                    match               = 1;
                    rigidTransformation = PutTogether(puzzle(skewMatrix(i,4)).edge{skewMatrix(i,5),1},puzzle(skewMatrix(i,2)).edge{skewMatrix(i,3),1}, ...
                        puzzle(skewMatrix(i,4)).piece, puzzle(skewMatrix(i,2)).piece);
                elseif ismember(skewMatrix(i,4),clusters{h,1}) && ismember(skewMatrix(i,2),clusters{p,1}) && skewMatrix(i,1)<1
                    match               = 1;
                    rigidTransformation = PutTogether(puzzle(skewMatrix(i,2)).edge{skewMatrix(i,3),1},puzzle(skewMatrix(i,4)).edge{skewMatrix(i,5),1}, ...
                        puzzle(skewMatrix(i,2)).piece, puzzle(skewMatrix(i,4)).piece);
                end
                
                %CALCULATING RIGID TRANSFORMATION MATRIX
                if match==1
                    if (rigidTransformation~='w')
                        rotation    = rigidTransformation(1:3,1:3);
                        translation = rigidTransformation(1:3,4);
                        
                        for j=1:length(cluster)
                            for k=1:length(puzzle(cluster(j)).piece(:,1))
                                puzzle(cluster(j)).piece(k,:) = (rotation*(puzzle(cluster(j)).piece(k,:))'+translation)';
                            end
                            for k=1:length(puzzle(cluster(j)).edge(:,1))
                                for l=1:length(puzzle(cluster(j)).edge{k,1}(:,1))
                                    puzzle(cluster(j)).edge{k,1}(l,:) = (rotation*(puzzle(cluster(j)).edge{k,1}(l,:))'+translation)';
                                end
                            end
                        end
                        break
                    else
                        continue
                    end
                end
            end
        end
    end
end

figure(3); hold on
for i=1:length(x)
    if isempty(sol(i).match)==0
        plot3(puzzle(i).piece(:,1), puzzle(i).piece(:,2),puzzle(i).piece(:,3),'LineWidth',2);
    end
end


out=puzzle;

end
%% PROBLEMS

% UPDATE MOVED AFTER SOLUTION UPDATE
% CHECK ADJACENT EDGES, IF MATCHING EDGES ARE SAME PIECE